# by joevt Feb 2, 2020

# dump disk_label file
dump_label () { local contents=$(xxd -p -c99999 $1); echo ${contents:10} | perl -pe "s/(..)/\1 /g;s/00/../g;s/ //g;s/(.{$((0x${contents:2:4}*2))})/\1\n/g" ; }


# mount a partition
mountpartition() {
	local mounttype=$1
	local slice=$2
	local volume=$3
	local mountpoint=$(mount | sed -n -E "/\/dev\/$slice on (.*) \(.*/s//\1/p")
	if [ -z $mountpoint ]; then
		i=0
		local startmountpoint="/Volumes/$volume"
		mountpoint="$startmountpoint"
		while [ -d "$mountpoint" ]; do
			((i++))
			mountpoint="$startmountpoint$i"
		done
	fi
	if [ ! -d "$mountpoint" ]; then
		sudo mkdir "$mountpoint" 2> /dev/null
		sudo mount$mounttype "/dev/$slice" "$mountpoint"
	fi
	echo -n "$mountpoint"
}


makemultilinedisklabel () {
	local folder="$1"
	local lines="$2"
	local alllines=""
	local alllines2x=""

	IFS=$'\n'
	local linenum=0
	local theline=""
	for theline in $(echo "$lines"); do
		sudo bless --folder "$folder" --label "$theline"
		if (( $linenum )); then
			alllines+="$(dump_label "$folder/.disk_label" | sed '1,/[^.]/ {/^[.]*$/d; }')"$'\n'
			alllines2x+="$(dump_label "$folder/.disk_label_2x" | sed '1,/[^.]/ {/^[.]*$/d; }')"$'\n'
		else
			alllines+="$(dump_label "$folder/.disk_label")"$'\n'
			alllines2x+="$(dump_label "$folder/.disk_label_2x")"$'\n'
		fi
		((linenum++))
	done

	local suffix=""
	local thelines=""
	for suffix in "" "_2x"; do
		[[ -z $suffix ]] && thelines="$alllines" || thelines="$alllines2x"
		local linewidths=$(echo "${thelines//[0-9a-f]/.}" | sort -ur)
		local maxlinewidth=$(echo "$linewidths" | sed -n '1p')
		linewidths=$(echo "$linewidths" | sed '1d')

		local centercommands=""
		local linewidth=""
		for linewidth in $(echo "$linewidths"); do
			local left=${maxlinewidth:0:(${#maxlinewidth}-${#linewidth})/4*2}
			local right=${maxlinewidth:0:(${#maxlinewidth}-${#linewidth}-${#left})}
			centercommands+="s/^(${linewidth})$/$left\1$right/; "
		done
		thelines=$(echo "$thelines" | sed -E "$centercommands s/\./0/g")
		[[ -d "/tmp/$folder" ]] || mkdir -p "/tmp/$folder"
		printf "01%04x%04x$thelines" $((${#maxlinewidth} / 2)) $(echo "$thelines" | wc -l) | xxd -p -r > "/tmp/$folder/disk_label$suffix"
		sudo cp "/tmp/$folder/disk_label$suffix" "$folder/.disk_label$suffix"
		[[ -f $folder/.disk_label.contentDetails ]] && sudo touch $folder/.disk_label.contentDetails
		#sudo bless --folder "$folder" --labelfile "/tmp/$folder/disk_label$suffix"
		#dump_label "$folder/.disk_label$suffix"
	done
}


getvolumename () {
	local slice="$1"
	eval "$(diskutil info -plist /dev/$slice | plutil -p - | sed -nE '/^ *"VolumeName" => (.*)/s//echo \1/p')"
}


mountEFIpartitions () {
	IFS=$'\n'
	local diskinfo=""
	for diskinfo in $(diskutil list | sed -nE '/^ *[0-9]+: +EFI (.*[^ ]) +[0-9.]+ [^ ]?B +(disk[0-9]+s[0-9]+)$/s//\1_\2/p'); do
		local slice="${diskinfo##*_}"
		local volume="$(getvolumename $slice)"
		mountpartition "_msdos" "$slice" "$volume" > /dev/null
	done
}


mountRecoveryHDpartitions() {
	IFS=$'\n'
	for diskinfo in $(diskutil list | sed -nE '/^ *[0-9]+: +Apple_Boot (Recovery HD) +[0-9.]+ [^ ]?B +(disk[0-9]+s[0-9]+)$/s//\1_\2/p'); do
		local slice="${diskinfo##*_}"
		local volume="$(getvolumename $slice)"
		mountpartition "_hfs" "$slice" "$volume" > /dev/null
	done
}


mountRecoverypartitions() {
	IFS=$'\n'
	for diskinfo in $(diskutil list | sed -nE '/^ *[0-9]+: +APFS Volume (Recovery) +[0-9.]+ [^ ]?B +(disk[0-9]+s[0-9]+)$/s//\1_\2/p'); do
		local slice="${diskinfo##*_}"
		local volume="$(getvolumename $slice)"
		mountpartition "_apfs" "$slice" "$volume" > /dev/null
	done
}


dumpAllDiskLabels () {
	# Only includes Apple_HFS and Apple_Boot partitions.
	# Partitions need to be mounted first.
	IFS=$'\n'
	for diskinfo in $(diskutil list | sed -nE '/^ *[0-9]+: +Apple_[HFSBoot]* ([^ ].*[^ ]) +[0-9.]+ [^ ]?B +(disk[0-9]+s[0-9]+)$/s//\1_\2/p'); do
		local slice="${diskinfo##*_}"
		local volume="$(getvolumename $slice)"
		local mountpoint=$(mountpartition "_hfs" "$slice" "$volume")
		local theblessinfo="$(bless --info "$mountpoint")"
		local thefolder=""
		local thefile=""
		local theopen=""
		eval $(echo "$theblessinfo" | sed -nE '/^finderinfo\[0\]: +[0-9]+ => Blessed System Folder is (.*)/s//thefolder="\1"/p; /^finderinfo\[1\]: +[0-9]+ => Blessed System File is (.*)/s//thefile="\1"/p;  /^finderinfo\[2\]: +[0-9]+ => 1st dir in open-folder list is (.*)/s//theopen="\1"/p; ')
		if [[ -f "$thefolder/SystemVersion.plist" ]]; then
			local version=$(/usr/libexec/PlistBuddy -c 'Print :ProductVersion' "$thefolder/SystemVersion.plist")
			echo "#$slice=$mountpoint, $volume ($version)"
			[[ -f $thefolder/.disk_label_2x ]] && dump_label "$thefolder/.disk_label_2x" || echo "    missing .disk_label_2x"
			[[ -f $thefolder/.disk_label ]] && dump_label "$thefolder/.disk_label" || echo "    missing .disk_label"
		fi
	done
}


makeLabelCommands () {
	# Only includes Apple_HFS and Apple_Boot partitions.
	# Partitions need to be mounted first.
	IFS=$'\n'
	for diskinfo in $(diskutil list | sed -nE '/^ *[0-9]+: +Apple_[HFSBoot]* ([^ ].*[^ ]) +[0-9.]+ [^ ]?B +(disk[0-9]+s[0-9]+)$/s//\1_\2/p'); do
		local slice="${diskinfo##*_}"
		local volume="$(getvolumename $slice)"
		local mountpoint=$(mountpartition "_hfs" "$slice" "$volume")
		local theblessinfo="$(bless --info "$mountpoint")"
		local thefolder=""
		local thefile=""
		local theopen=""
		eval $(echo "$theblessinfo" | sed -nE '/^finderinfo\[0\]: +[0-9]+ => Blessed System Folder is (.*)/s//thefolder="\1"/p; /^finderinfo\[1\]: +[0-9]+ => Blessed System File is (.*)/s//thefile="\1"/p;  /^finderinfo\[2\]: +[0-9]+ => 1st dir in open-folder list is (.*)/s//theopen="\1"/p; ')
		if [[ -f "$thefolder/SystemVersion.plist" ]]; then
			local version=$(/usr/libexec/PlistBuddy -c 'Print :ProductVersion' "$thefolder/SystemVersion.plist")
			echo "#$slice=$mountpoint, $volume ($version)"
			echo makemultilinedisklabel \""$thefolder"\" \""${volume/Recovery HD/Recovery}\"\$'\\\n'\"$version\""
		fi
	done
}


clearOpenFolders () {
	# Clear open folder list of hfs partitions (finderinfo[2] in bless --info).
	# If you have an HFS+ disk that has folders that open when you mount them (such as an installer), then this will stop that from happening.
	IFS=$'\n'
	for diskinfo in $(diskutil list | sed -nE '/^ *[0-9]+: +Apple_HFS (.*[^ ]) +[0-9.]+ [^ ]?B +(disk[0-9]+s[0-9]+)$/s//\1_\2/p'); do
		local slice="${diskinfo##*_}"
		local volume="$(getvolumename $slice)"
		local mountpoint=$(mountpartition "_hfs" "$slice" "$volume")
		echo "#$slice=$mountpoint,$volume ($version)"
		local theblessinfo="$(bless --info "$mountpoint")"
		local thefolder=""
		local thefile=""
		local theopen=""
		eval $(echo "$theblessinfo" | sed -nE '/^finderinfo\[0\]: +[0-9]+ => Blessed System Folder is (.*)/s//thefolder="\1"/p; /^finderinfo\[1\]: +[0-9]+ => Blessed System File is (.*)/s//thefile="\1"/p;  /^finderinfo\[2\]: +[0-9]+ => 1st dir in open-folder list is (.*)/s//theopen="\1"/p; ')
		if [[ -n $theopen ]]; then
			echo "Clearing open folder list"
			sudo bless --folder "$thefolder" --file "$thefile"
		fi
	done
}
